#!/bin/bash
# Prends en argument k n topologie
# Exemple : ./search_config.sh 3 7 ring


set -x
if [ $# -ne 3 ]
    then echo "3 arguments are expected"
    echo "$0 <k> <n> <topology filename without extension"
    echo "Exemple : ./search_config.sh 3 7 ring"
    exit 1
fi

k=$1
n=$2
topology=$3
log=Log/$topology${k}_$n.log


##############################################################
# building the org table of results using the Log/*.log files

valid=""
invalid=""
timeout=""
stack_overflow=""

grep "ok: valid" $log && valid="true"
grep "ok: invalid" $log && invalid="true"
grep "Wallclock timeout" $log && timeout="true"
grep "Stack overflow" $log && stack_overflow="true"

time=`grep "total runtime" $log | cut -d ' ' -f5`

if [ $valid = "true" ]
    then resultat="legitimate"
    step=`grep "ok: " $log | cut -d ' ' -f4 | tr -d ')'`
fi

if [ "$invalid" = "true" ]
    then resultat="cycle found"
    step=`grep "ok: " $log | cut -d ' ' -f4`
fi

if [ "$timeout" = "true" ]
    then time="TIMEOUT"
    step=`grep "ok: " $log | cut -d ' ' -f5`
fi

if [ "$valid$invalid$timeout" = "" ]
    then resultat="..."
fi

# nb: a SO does not mean that kind2 is over
if  [ "$valid$invalid" = "" -a "$stack_overflow" = "true" ]
    then resultat="$resultat (SO)"
fi


# d=$(expr $n / 2)
d=$(expr $n / 2)
dx2m1=$(expr $d \* 2 - 1)
dx3m1=$(expr $d \* 3 - 1)


echo "| $topology | $k ($dx2m1) | $n | $resultat | $step | $dx3m1 | $time |"  > $log.org

